package io.fritz2.examples.remote

import io.fritz2.binding.RootStore
import io.fritz2.binding.const
import io.fritz2.dom.html.html
import io.fritz2.dom.mount
import io.fritz2.dom.values
import io.fritz2.remote.body
import io.fritz2.remote.onErrorLog
import io.fritz2.remote.remote
import kotlinx.coroutines.ExperimentalCoroutinesApi
import kotlinx.coroutines.FlowPreview

@ExperimentalCoroutinesApi
@FlowPreview
fun main() {

    val userStore = object : RootStore<String>("") {

        val users = remote("https://reqres.in/api/users").acceptJson()

        val loadAllUsers = apply<Unit, String> {
            users.get()
                .onErrorLog()
                .body()
        } andThen update

        val loadUserById = apply { s: String ->
            users.get(s)
                .onErrorLog()
                .body()
        } andThen update

        val saveUserWithName = apply { s: String ->
            users.post(
                body =
                """
                    {
                        "name": "$s",
                        "job": "programmer"
                    }
                """.trimIndent()
            )
                .onErrorLog()
                .body()
        } andThen update

    }

    html {
        div {
            div("form-group") {
                label("load-user") {
                    text("Load user by id")
                }
                input("form-control", id = "load-user") {
                    placeholder = const("Enter user id")
                    userStore.loadUserById <= changes.values()
                }
            }

            hr("my-4") { }

            div("form-group") {
                label("save-user") {
                    text("Save user")
                }
                input("form-control", id = "save-user") {
                    placeholder = const("Enter new user name")
                    userStore.saveUserWithName <= changes.values()
                }
            }

            hr("my-4") { }

            div("form-group") {
                button("btn btn-primary") {
                    text("Load all users")
                    userStore.loadAllUsers <= clicks
                }
            }
            div("card card-body") {
                h6("card-title") {
                    text("User store data")
                }
                pre("text-wrap") {
                    code {
                        userStore.data.bind()
                    }
                }
            }
        }
    }.mount("target")
}
